/* ccgo: go/go.hh
 *
 * Copyright 2003 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef GO_GO_HH
#define GO_GO_HH

#include <vector>

namespace go
{
	class Board;
	class Rule;
	class Node;
	class Game;

	// implement: simple square board
	class Board
	{
	public:
		enum State {
			STATE_EMPTY = 0,
			STATE_BLACK = 1,
			STATE_WHITE = 2
		};
		Board(int size);
		virtual ~Board();
		virtual State state(int loc);
		virtual void state(int loc, State);
		virtual std::vector<int> neighbors(int loc);
		int loc_range();
	private:
		int z;     // size
		State * b; // board
	};

	class Rule
	{
		Game & g;
		std::vector<int> kos;
	public:
		Rule(Game & game);
		virtual bool check(int, std::vector<int>);
		virtual void regis();
		virtual void unreg();
	};

	class Move
	{
	public:
		int loc;
		Board::State s;
		std::vector<int> e; // erased stones
		Board::State es;
	};

	class Node
	{
	public:
		Node();
		virtual ~Node();
		Move * move;
		Node * up;
		std::vector<Node *> down;
	};

	class Game
	{
		Node * root;
	public:
		enum Turn {
			TURN_BLACK = 1,
			TURN_WHITE = 2
		};
		Game();
		virtual ~Game();

		Node * n;
		Board * b;
		Rule * r;
		Turn turn;
		int num;

	        virtual Node * new_node();
		void set_board(Board * board);
		void set_rule(Rule * rule);
		virtual void init();
		virtual void add_move(int loc);

		virtual void up();
		virtual void down(int n);

		// housekeeping function
		void get_kill(int loc, std::vector<int> & k);
		int qi_group(int loc, std::vector<int> & g, Board::State, bool * mark);
	};
}
#endif // GO_GO_HH
