/* ccgo: go/textgo.cc
 *
 * Copyright 2003 Chun-Chung Chen <cjj@u.washington.edu>
 *
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <textgo.hh>

using namespace text;
using namespace std;

Board::Board(int z) :
	go::Board(z)
{
	ln = z;
	text = new char[ln * ln];
	for (int i = 0; i < ln * ln; i ++) state(i, STATE_EMPTY);
}

Board::~Board()
{
	delete text;
}

go::Board::State Board::state(int l)
{
	switch (text[l]) {
	case '#':
		return STATE_BLACK;
	case 'O':
		return STATE_WHITE;
	default:
		return STATE_EMPTY;
	}
}

void Board::state(int l, go::Board::State s)
{
	char c;
	switch (s) {
	case STATE_EMPTY:
		c = '.';
		if (ln > 6) {
			bool side0 = l % ln == 3 || l % ln == ln - 4;
			bool side1 = l / ln == 3 || l / ln == ln - 4;
			bool mid0 = ln % 2 && l % ln == ln / 2;
			bool mid1 = ln % 2 && l / ln == ln / 2;
			if (ln > 13) {
				if (side0 && side1 || side0 && mid1 || mid0 && side1 || mid0 && mid1) c = '+';
			} else if (ln > 9) {
				if (side0 && side1 || mid0 && mid1) c = '+';
			} else if (ln > 7) {
				side0 = l % ln == 2 || l % ln == ln - 3;
				side1 = l / ln == 2 || l / ln == ln - 3;
				if (side0 && side1 || mid0 && mid1) c = '+';
			} else {
				if (mid0 && mid1) c = '+';
			}
		}
		text[l] = c;
		break;
	case STATE_BLACK:
		text[l] = '#';
		break;
	case STATE_WHITE:
		text[l] = 'O';
		break;
	}
}

vector<int> Board::neighors(int l)
{
	vector<int> n;
	if (l % ln > 0) n.push_back(l - 1);
	if (l % ln < ln - 1) n.push_back(l + 1);
	if (l / ln > 0) n.push_back(l - ln);
	if (l / ln < ln - 1) n.push_back(l + ln);
	return n;
}

void Board::show(ostream & o)
{
	o << "   ";
	for (int i = 0; i < ln; i ++) {
		char c = 'a' + i;
		if (c >= 'i') c ++;
		if (i > 0) o << ' ';
		o << c;
	}
	o << '\n';
	for (int i = 0; i < ln; i ++) {
		int c = 19 - i;
		if (c < 10) o << ' '; 
		o << c << '|';
		for (int j = 0; j < ln; j ++) {
			if (j > 0) o << ' ';
			o << text[i * ln + j];
		}
		o << '|';
		if (c < 10) o << ' '; 
		o << c;
		o << "\n";
	}
	o << "   ";
	for (int i = 0; i < ln; i ++) {
		char c = 'a' + i;
		if (c >= 'i') c ++;
		if (i > 0) o << ' ';
		o << c;
	}
	o << '\n';
}

// members of class text::Game
void Game::init()
{
	set_board(new Board(19));
	go::Game::init();
}

void Game::show(ostream & o)
{
	cout << "move # " << num << endl;
	switch (turn) {
	case TURN_BLACK:
		cout << "black to play" << endl;
		break;
	case TURN_WHITE:
		cout << "white to play" << endl;
		break;
	}
	cout << "down number = " << n->down.size() << endl;
	if (! b) {
		cout << "no board" << endl;
		return;
	} else {
		dynamic_cast<Board *>(b)->show(o);
	}
}
